function y = blkdiag(varargin)

% BLKDIAG  Block diagonal concatenation of input arguments.
%
%                                           |A1 0 .. 0|
%   Y = BLKDIAG2(A1,A2,..,An)  produces     |0 A2 .. 0|
%                                           |0 0  .. An|
%
%   See also DIAG, HORZCAT, VERTCAT

% Modified version of blkdiag.m
% Bjarne Grimstad 08/03/09

if (nargin < 1)
    display('Too few arguments! Please supply at least one argument.');
elseif (nargin == 1)
    y = varargin{1};
else
    
    % Calculates the size of y
    nx = 0; ny = 0;
    for i = 1:nargin
        nx = nx + size(varargin{i},1);
        ny = ny + size(varargin{i},2);
    end
    y = zeros(nx,ny);
    
    % Fills out the blocks on the diagonal
    xnow = 1; ynow = 1;
    for j = 1:nargin
        y(xnow:xnow+size(varargin{j},1)-1, ynow:ynow+size(varargin{j},2)-1) = varargin{j};
        xnow = xnow+size(varargin{j},1);
        ynow = ynow+size(varargin{j},2);
    end
    
end
