function Y = blkdiag2(varargin)

% BLKDIAG2  Block diagonal concatenation of input arguments.
%
%                                   |A 0 .. 0|
%   Y = BLKDIAG2(A,N)  produces     |0 A .. 0|
%                                   |0 0 .. A|
%   where 	size(A) = [m,n]
%			size(Y) = [N*m,N*n]
%
%   See also DIAG, HORZCAT, VERTCAT


A = varargin{1};
N = varargin{2};
[m,n] = size(A);

Y = zeros(m*N,n*N);
for k = 1:N
  Y(m*(k-1)+1:m*k,n*(k-1)+1:n*k) = A;
end
