function [ M ] = calcM( sysStruct, probStruct, K )
%CALCM Calculates M(K)
%   [M] = calcM( sysStruct, probStruct, K )
%
%   Outputs:
%   M           - The closed-loop objective function 
%                 M = (Ac^N)'P(Ac^N) + SUM(i=0,N-1) (Ac^i)'(Q + C'K'RKC)(Ac^i)
%                 where Ac = A - BKC
%
%   Inputs:
%   sysStruct   - System struct (discrete). Can be generated using ss().
%   probStruct  - Problem struct holding Q,R,P,N
%   K           - Gain matrix (Negative feedback, i.e. u = -Ky)

% Extracts model
A = sysStruct.A;
B = sysStruct.B;
C = sysStruct.C;

% Extracts tuning matrices
Q = probStruct.Q;
R = probStruct.R;
P = probStruct.P;
N = probStruct.N;

% Calculates M(K)
W = Q + (K*C)'*R*(K*C);
Atilde = (A - B*K*C);
Atemp = Atilde^0;

M = Atemp'*P*Atemp;
for i = 0:N-1
   M = M + Atemp'*W*Atemp;
   Atemp = Atemp*Atilde;
end

M = M + Atemp'*P*Atemp;

end
