function [ P ] = calcP( sysStruct, probStruct)
%CALCP2 Finds the generalized plant
%     |    A    | W^(1/2)     0       B    |
%     --------------------------------------
% P = | Q^(1/2) |    0        0       0    |
%     |    0    |    0        0    R^(1/2) |
%     |    C    |    0     V^(1/2)    0    |
%
% Note: Assumes D=0

[nx,nu,ny] = sysdim(sysStruct.A, sysStruct.B, sysStruct.C);
Agen = sysStruct.A;
Bgen = [sqrtm(probStruct.W) zeros(nx,ny) sysStruct.B];
Cgen = [sqrtm(probStruct.Q); 
        zeros(nu,nx); 
        sysStruct.C];
Dgen = [zeros(nx, nx+ny+nu); 
        zeros(nu,nx+ny) sqrtm(probStruct.R); 
        zeros(ny,nx) sqrtm(probStruct.V) zeros(ny,nu)];
P = ss(Agen,Bgen,Cgen,Dgen,sysStruct.Ts);
% P = tf(P);

end

