function [ Fl ] = clnorm( P, K )
%CLNORM Returns the closed-loop 2-norm of a negative feedback system
%   Closed-loop 2-norm: ||Fl(P,K)||^2
%   Where Fl(P,K) = lft(P,K)

% Calculates the lower LFT of P and K
Plft = lft(P,K);

% When the lft of a discrete system is calculated Matlab forgets to set Ts
if (P.Ts ~= 0)
    Plft.Ts = P.Ts; % Updates the sampling time
end

% Closed-loop norm
Fl = norm(Plft,2);

% Numerical problem with Inf
threshold = 1000; % Problem specific
if (Fl > threshold)
    Fl = threshold; 
end
% Used threshold values:
% 1000 for uDAQ, 
% 10 for second-order, 
% 50 for convexity example
end

