% This example illustrates that even simple systems may yield a non-convex
% closed-loop optimization problem
close all
clear all
clc

warning off all

% Plant model
A = [0.9 -0.1; 
    0.1 0.7]; 
B = [-0.5 0.1; 
    -0.7 0.6]; 
C = [1 1]; 
D = zeros(1,2);
Ts = 1;
system = ss(A,B,C,D,Ts);
[nx,nu,ny] = sysdim(A,B,C);

% eig(A)
% isControllable(A,B)
% isObservable(A,C)

% Performance weights
Q = diag([1 1]);
R = 0.1*diag([1 1]);
P = zeros(2); %P = dlyap(A',Q);
W = eye(nx);
V = zeros(ny);
N = 100;
problem = probStruct(Q,R,P,N,W,V);

a11 = [0.8 1.2]; a22 = [0.6 0 1.2];
a12 = [-.5 0 .5]; a21 = a12;

b11 = [-1 1]; b22 = b11;
b12 = [-0.5 0 0.5]; b21 = b12;

for k_a11 = 1:length(a11)
for k_a12 = 1:length(a12)
for k_a21 = 1:length(a21)
for k_a22 = 1:length(a22)

for k_b11 = 1:length(b11)
for k_b12 = 1:length(b12)
for k_b21 = 1:length(b21)
for k_b22 = 1:length(b22)

A = [a11(k_a11) a12(k_a12); 
    a21(k_a21) a22(k_a22)]; 
B = [b11(k_b11) b12(k_b12); 
    b21(k_b21) b22(k_b22)]; 
C = [1 1]; 
D = zeros(1,2);
Ts = 1;
system = ss(A,B,C,D,Ts);
    
% generalized plant
P = calcP(system,problem);

% LQR controller. Used to find a suboptimal controller
% Klqr = dlqr(A, B, Q, R); 

% Data vectors
k1 = -3:0.2:3;
k2 = -3:0.2:3;
m = length(k1);
n = length(k2);

K = [0; 0];
Jvec = zeros(m,n);
Mvec = zeros(m,n);

disp('Finding objective functions...');
for i = 1:m
    for j = 1:n
        K(1) = k1(i);
        K(2) = k2(j);
        Jvec(i,j) = clnorm(P,-K);
        %Mvec(i,j) = trace(calcM(system,problem,K));
    end
end

% disp('Plotting objective functions...');
% figure
% mesh(k2,k1,Jvec);
% xlim([k2(1) k2(end)]);
% ylim([k1(1) k1(end)]);
% zlim([0 100]);
% 
% figure
% mesh(k2,k1,Mvec);
% zlim([0 100]);

% save data
timenow = datestr(now, 'dd-mm-yyyy_HH-MM-SS');
filename = ['./data/Jcl_', timenow];
%filename2 = ['./data/Mcl_', timenow];
save(filename, 'Jvec', 'system', 'problem');
% save(filename2, 'Mvec', 'system', 'problem');

end
end
end
end
end
end
end
end
