clc
clear all
close all

% The general second-order system is described as 
% g(s) = k/(tau^2 s^2 + 2 tau zeta s + 1)
% where tau = 1 and zeta = [0,1]
% zeta < 1 gives an underdamped system
k = 1;
tau = 1;
zeta = 0.7;
g = tf(k, [tau^2, 2*tau*zeta, 1]);
G = ss(g); eig(G)
A = G.A; B = G.B;
C = G.C; D = G.D;
[nx,nu,ny] = sysdim(A,B,C);

% Discretization
% Note that d2c with zoh does not handle poles at zero z = 0, or negative
% real poles
Ts = 0.1;
Gdisc = c2d(G,Ts); eig(Gdisc)
Ad = Gdisc.A; Bd = Gdisc.B;
Cd = Gdisc.C; Dd = Gdisc.D;

% Problem formulation
Q = Cd'*diag([1])*Cd; %Q = diag([1 1]);
Q = eye(2);
R = 1;
N = 100; % Horizon length
P = Q;

%% PI controller
Api = [A zeros(nx,ny);
       C zeros(ny,ny)
       ];
Bpi = [B; D];
Cpi = blkdiag(C, eye(ny));
Dpi = [D; zeros(ny,nu)];

Gpi = ss(Api,Bpi,Cpi,Dpi); eig(Gpi)
Gpi = c2d(Gpi, Ts); % Discretizes

Api = Gpi.A; Bpi = Gpi.B;
Cpi = Gpi.C; Dpi = Gpi.D;

[nx_pi, nu_pi, ny_pi] = sysdim(Gpi.A, Gpi.B, Gpi.C);

Qpi = blkdiag(Q,0);
% Note: if the integral action is not weighted, the marginal poles are not
% moved and the norm of the closed-loop system becomes Inf!
Rpi = R;
Ppi = Qpi; %Ppi = dlyap(Api',Qpi);
Ppi = zeros(3);
probPI = probStruct(Qpi,Rpi,Ppi,N);
genPpi = calcP(Gpi,probPI);

Kp = -1:0.1:10;
Ki = -1:0.1:8;
J = zeros(length(Kp), length(Ki));
for i = 1:length(Kp)
    for j = 1:length(Ki)
        K = [Kp(i) Ki(j)];
        J(i,j) = clnorm(genPpi, -K);
        if (J(i,j) > 50)
            J(i,j) = 50;
        end
    end
end

figure()
mesh(Ki,Kp,J);

figure()
contour(Ki,Kp,J,1:1:1000);

