% This example illustrates that even simple SISO systems may yield a 
% non-convex closed-loop optimization problem
close all
clear all
clc

warning off all

%load('./data/SISO_Jcl_05-07-2009_02-07-17.mat');
load('./data/SISO_Jcl_05-07-2009_02-08-08.mat');
Ts = 1;
system.Ts = Ts;
system.A = -(system.A);
[A,B,C,D] = ssdata(system);

A = [0.31 0.45 0.20;
    0.33 1.11 0.10;
    0 -0.51 0.15];

B = [0 0.69 0.38]';

C = [-1.0 0.9 -0.8];

D = 0;

system = ss(A,B,C,D,Ts);

[nx,nu,ny] = sysdim(A,B,C);



% G = tf(system);
% figure
% bode(system)

eig(A)
isControllable(A,B)
isObservable(A,C)

% Performance weights
%Q = eye(nx);
Q = eye(nx);
R = 0.01;
P = zeros(nx); %
P = dlyap(A',Q);
%P = Q;
W = eye(nx);
V = zeros(ny);
N = 100;
problem = probStruct(Q,R,P,N,W,V);

problem1 = probStruct(Q,R,P,120,W,V);
problem2 = probStruct(Q,R,P,200,W,V);
problem3 = probStruct(Q,R,P,400,W,V);

% generalized plant
P = calcP(system,problem);

% LQR controller. Used to find a suboptimal controller
% Klqr = dlqr(A, B, Q, R); 

% Data vectors
%K = -10:0.1:10;
K = [-2:0.05:12]; % works for the second system
% K = [0:10:1000 1000:100:1e5]; % works for the first system
m = length(K);

Jvec = zeros(m,1);
Mvec = zeros(m,1);

Mvec1 = zeros(m,1);
Mvec2 = zeros(m,1);
Mvec3 = zeros(m,1);

% disp('Finding objective functions...');
for i = 1:m
   Jvec(i) = clnorm(P,-K(i));
   Mvec(i) = sqrt(norm(trace(calcM(system,problem,K(i))),2));
   if Mvec(i) > 50
       Mvec(i) = 50;
   end
   
   Mvec1(i) = sqrt(norm(trace(calcM(system,problem1,K(i))),2));
   if Mvec1(i) > 50
       Mvec1(i) = 50;
   end
   
   Mvec2(i) = sqrt(norm(trace(calcM(system,problem2,K(i))),2));
   if Mvec2(i) > 50
       Mvec2(i) = 50;
   end
   
   Mvec3(i) = sqrt(norm(trace(calcM(system,problem3,K(i))),2));
   if Mvec3(i) > 50
       Mvec3(i) = 50;
   end
   
end

minima = find_minima(Jvec)

if minima > 1
    
    disp('Found a non-convex cost function! Saving data...');
    
    % save data
    timenow = datestr(now, 'dd-mm-yyyy_HH-MM-SS');
    filename = ['./data/SISO_Jcl_', timenow];
    %save(filename, 'Jvec', 'system', 'problem');
end

disp('Plotting objective functions...');
figure
plot(K, Jvec, 'Linewidth', 1.5);
xlim([K(1) K(end)]);
ylim([0 51]);
xlabel('Controller gain');
ylabel('Cost function');
l = legend('$J_{cl}$');
set(l, 'Interpreter', 'Latex')
set(l, 'FontSize', 12)

figure
plot(K, Jvec, 'Linewidth', 2);
hold on
plot(K, Mvec3, '--k', 'Linewidth', 1.5);
plot(K, Mvec2, '-.k', 'Linewidth', 1.5);
plot(K, Mvec1, ':k', 'Linewidth', 1.5);
hold off
xlim([K(1) K(end)]);
ylim([0 51]);
xlabel('Controller gain');
ylabel('Cost function');
l = legend('$J_{cl}$');
set(l, 'Interpreter', 'Latex')
set(l, 'FontSize', 12)
