% This example illustrates that even simple SISO systems may yield a 
% non-convex closed-loop optimization problem
close all
clear all
clc

warning off all

% A = [0.9 -0.1 0.2; 
%     0.1 0.7 -.4;
%     0 1 -0.2]; 
% B = [-0.5 0.3 0.1]'; 
% C = [1 1 1]; 
% D = 0;
% Ts = 1;
% system = ss(A,B,C,D,Ts);
% [nx,nu,ny] = sysdim(A,B,C);

% eig(A)
% isControllable(A,B)
% isObservable(A,C)

% Plant model
a11 = [-0.31 0.78 1.2]; a12 = [-0.45 0 0.42]; a13 = [-0.2 0 0.63];
a21 = [-0.33 0 0.28]; a22 = [-1.11 -0.23 0.96]; a23 = [-0.1 0 1.33];
a31 = [-0.3 0 0.53]; a32 = [-0.77 0 0.51]; a33 = [-0.15 0 1.07];
Na = 3;

b1 = [-1 0 0.5]; b2 = [-0.4 0 0.69]; b3 = [-1.11 0 0.38];
Nb = 3;

c1 = [-1 0 1]; c2 = [-0.9 0 0.9]; c3 = [-0.8 0 0.8];
Nc = 3;

for ka11 = 1:Na
for ka12 = 1:Na
for ka13 = 1:Na
for ka21 = 1:Na
for ka22 = 1:Na
for ka23 = 1:Na
for ka31 = 1:Na
for ka32 = 1:Na
for ka33 = 1:Na

for kb1 = 1:Nb
for kb2 = 1:Nb
for kb3 = 1:Nb    

for kc1 = 1:Nc
for kc2 = 1:Nc
for kc3 = 1:Nc    
    
A = [a11(ka11) a12(ka12) a13(ka13);
    a21(ka21) a22(ka22) a23(ka23);
    a31(ka31) a32(ka32) a33(ka33)
    ];

B = [b1(kb1) b2(kb2) b3(kb3)]';

C = [c1(kc1) c2(kc2) c3(kc3)];

D = 0;

Ts = 1;
system = ss(A,B,C,D,Ts);
[nx,nu,ny] = sysdim(A,B,C);

% Performance weights
Q = diag([1 1 1]);
R = 0.01;
P = zeros(nx); %P = dlyap(A',Q);
W = eye(nx);
V = zeros(ny);
N = 100;
problem = probStruct(Q,R,P,N,W,V);
    
% generalized plant
P = calcP(system,problem);

% LQR controller. Used to find a suboptimal controller
% Klqr = dlqr(A, B, Q, R); 

% Data vectors
K = -5:0.1:5;
m = length(K);

Jvec = zeros(m,1);
Mvec = zeros(m,1);

% disp('Finding objective functions...');
for i = 1:m
   Jvec(i) = clnorm(P,-K(i));
   %Mvec(i) = trace(calcM(system,problem,K(i)));
end

% Find number of minima
minima = find_minima(Jvec);

if minima > 1
    
    disp('Found a non-convex cost function! Saving data...');
    
    % save data
    timenow = datestr(now, 'dd-mm-yyyy_HH-MM-SS');
    filename = ['./data/SISO_Jcl_', timenow];
    save(filename, 'Jvec', 'system', 'problem');
end

end
end
end
end
end
end
end
end
end
end
end
end
end
end
end

% disp('Plotting objective functions...');
% figure
% plot(K,Jvec);
% xlim([K(1) K(end)]);
% ylim([0 1010]);

% figure
% plot(K,Mvec);
% xlim([K(1) K(end)]);
% ylim([0 1010]);



