% Here we show that the cost function J1 = ||F_l(P,K)||_2 and its 
% approximation J2 = trace(M(K)) are the same for some examples

close all
clear all
clc

warning off all

% Plant model
A = [0.8 0; 
    0 -0.6]; 
B = [1 -0.2; 0.1 1]; 
C = [0.8 0.7]; 
D = zeros(1,2);
Ts = 1;
system = ss(A,B,C,D,Ts);
[nx,nu,ny] = sysdim(A,B,C);

% Performance weights
Q = diag([1 1]);
R = 0.1*diag([1 1]);
P = zeros(2); %
P = dlyap(A',Q);
W = eye(nx);
V = zeros(ny);
N = 100;
problem = probStruct(Q,R,P,N,W,V);

% generalized plant
P = calcP(system,problem);

% LQR controller. Used to find a suboptimal controller
% Klqr = dlqr(A, B, Q, R); 

% Data vectors
k1 = -3:0.1:3;
k2 = -3:0.1:3;
m = length(k1);
n = length(k2);

K = [0; 0];
Jvec = zeros(m,n);
Mvec = zeros(m,n);

disp('Finding objective functions...');
for i = 1:m
    for j = 1:n
        K(1) = k1(i);
        K(2) = k2(j);
        Jvec(i,j) = clnorm(P,-K);
        
        Mvec(i,j) = sqrt(trace(calcM(system,problem,K)));
        if Mvec(i,j) > 50 % threshold
            Mvec(i,j) = 50;
        end
    end
end

disp('Plotting objective functions...');
figure
mesh(k2,k1,Jvec);
xlim([k2(1) k2(end)]);
ylim([k1(1) k1(end)]);
zlim([0 51]);

figure
mesh(k2,k1,Mvec);
xlim([k2(1) k2(end)]);
ylim([k1(1) k1(end)]);
zlim([0 51]);

figure
mesh(k2,k1,Jvec-Mvec);
xlim([k2(1) k2(end)]);
ylim([k1(1) k1(end)]);
zlim([-10 51]);

Jdiff = Jvec-Mvec;
Jsum = 0;
for i = 1:size(Jdiff,1)
    for j = 1:size(Jdiff,2)
        Jsum = Jsum + Jdiff(i,j);
    end
end
Jsum
