%% RGA analysis of the uDAQ28/LT

% Loads model
udaq_model;
[nx,nu,ny] = sysdim(A,B,C);
P = d2c(udaq_sys);
%[A,B,C,D] = ssdata(P);

N = 100;

G = zeros(ny,nu,N + 1);
RGA = zeros(ny,nu,N + 1);

omega_max = pi/Ts; % Nyquist frequency
omega = logspace(-4,log10(omega_max),N); % Frequencies from 10^(-1) to 10^1
disp('Computing RGA...');

% Continuous system
% for k = 1:N
%     G(:,:,k) = C * inv(omega(k) * 1i * eye(nx) - A) * B + D;
%     RGA(:,:,k) = G(:,:,k) .* pinv(G(:,:,k))';
% end

% Discrete system
for k = 1:N
    G(:,:,k) = C * inv(exp(1i*omega(k)*Ts) * eye(nx) - A) * B + D;
    RGA(:,:,k) = G(:,:,k) .* pinv(G(:,:,k))';
end

RGA_new = zeros(nu * ny, size(RGA,3));

c = 1;
for k = 1:ny
    for j = 1:nu
        for u = 1:size(RGA,3)
            RGA_new(c,u) = abs(RGA(k,j,u));
        end
        c = c + 1;
    end
end
RGA = RGA_new;

%% Plots RGA elements
close all;

figure();
semilogx(omega,RGA(1,1:N),'r-', 'Linewidth', 1.5)
hold on
semilogx(omega,RGA(2,1:N),'b-', 'Linewidth', 1.5)
semilogx(omega,RGA(3,1:N),'g-', 'Linewidth', 1.5)
semilogx(omega,RGA(4,1:N),'r-.', 'Linewidth', 1.5)
semilogx(omega,RGA(5,1:N),'b-.', 'Linewidth', 1.5)
semilogx(omega,RGA(6,1:N),'g-.', 'Linewidth', 1.5)
hold off
xlim([omega(1) omega(end)]);
ylim([-.01 1]);
legend('$|\lambda_{11}|$','$|\lambda_{12}|$','$|\lambda_{13}|$','$|\lambda_{21}|$','$|\lambda_{22}|$','$|\lambda_{23}|$');
ylabel('Magnitude')
xlabel('Frequency [rad/s]');

%% Plots singular values
sigmas = sigma(udaq_sys,omega);
sigmas = 20*log10(sigmas); % dB
figure();
semilogx(omega,sigmas(1,:),'k-', 'Linewidth', 1.5);
hold on;
semilogx(omega,sigmas(2,:),'k--', 'Linewidth', 1.5);
xlim([omega(1) omega(end)]);
%ylim([-.01 1]);
legend('$\overline{\sigma}$', '$\underline{\sigma}$');
ylabel('Magnitude [dB]')
xlabel('Frequency [rad/s]');
