clc
close all

load y_temp     % Temperature measurement sequence
load y_light    % Light measurement sequence
load Hp_filter2  % High pass filter, equiripple FIR filter (fdatool)
load Hp_filter3 % High pass filter, equiripple FIR filter (w_stop: 0.3, w_pass: 0.4)

% Extracts steady-state data
y1 = y_temp.signals.values(1:end,2);    % Temp inside system (direct)
y1_f = y_temp.signals.values(1:end,3);  % Temp inside system (filtered)
y2 = y_light.signals.values(1:end,1);   % Light intensity (direct)
y2_f = y_light.signals.values(1:end,2); % Light intensity (filtered)

% Picks out 10 minutes (sequence is 1 hour long)
% y1 = y1((1/Ts)*45*60:end-(1/Ts)*5*60); y1 = y1-2; %adjusts steady-state value
% y2 = y2((1/Ts)*45*60:end-(1/Ts)*5*60);

y1 = y1((1/Ts)*50*60:end-(1/Ts)*0*60); y1 = y1-2; %adjusts steady-state value
y2 = y2((1/Ts)*50*60:end-(1/Ts)*0*60);

N = length(y1); % Number of samples
Ts = 0.5;       % Sampling time
Fs = 1/Ts;      % Sampling frequency
time = [0:Ts:(N-1)*Ts]/60; % Time in minutes

% Plots the measurement sequence
figure
subplot(211)
plot(time,y1-mean(y1));
%ylim([37.8 38.3]);
xlim([0,time(end)]);
xlabel('Time [min]');
ylabel('Temperature [\circ C]');
legend('y_1');
subplot(212)
plot(time,y2-mean(y2));
xlim([0,time(end)]);
%ylim([21.4, 21.55]);
xlabel('Time [min]');
ylabel('Light intensity [-]');
legend('y_2');

%% Plots FFT of signal
y1 = detrend(y1);    % Removes linear trends
y2 = detrend(y2);

L = N;
NFFT = 2^nextpow2(L); % Next power of 2 from length of y
Y1 = fft(y1,NFFT)/L;
Y2 = fft(y2,NFFT)/L;
f = Fs/2*linspace(0,1,NFFT/2+1);

% Plot single-sided amplitude spectrum.
figure
semilogy(f,2*abs(Y1(1:NFFT/2+1)))
legend('Amplitude Spectrum of y_1')
xlabel('Frequency [Hz]')
ylabel('|Y_1(f)|')

ax = axis;
axis([ax(1:2) 1e-4 1])

figure
semilogy(f,2*abs(Y2(1:NFFT/2+1)))
legend('Amplitude Spectrum of y_2(t)')
xlabel('Frequency (Hz)')
ylabel('|Y_2(f)|')

ax = axis;
axis([ax(1:2) 1e-4 1])

% We see from the power spectral density that the noise lies on frequencies
% above ~0.1 Hz. We also note that the noise seems to have a peak at the frequency 0.91.
% We now try to create a filter that removes these frequencies

%% High pass filtering of meas sequence
figure
freqz(Hp_filter3);
xlabel('Frequency [Hz]');

% Runs the measurement sequence through the high pass filter
%y1_hpf = filter(Bhp,Ahp,y1);
y1_hpf = filter(Hp_filter3,1,y1);

figure
plot(time,y1_hpf);
ylim([-0.1 0.1]);
xlim([0 time(end)]);
xlabel('Time [min]');
ylabel('Temperature [\circ C]');
legend('High-pass filtered temperature measurement');

var(y1)
std(y1) %0.0182
var(y2)
std(y2) %0.0130

% From the high-pass filtered measurement sequences we get a mean amplitude
% of alpha_y1 = 0.025 and alpha_y2 = 0.01
