function [ ] = plotUdaqData2( y1, y2, u1, u2, r1, r2, time, Ts )
%PLOTUDAQDATA Plots data retrieved from UDAQ simulation
%   Inputs: y1: temperature measurement, 
%           y2: light intensity measurement,
%           u1: bulb input, 
%           u2: fan input, 
%           u3: LED input,
%           time: time vector in seconds,
%           Ts: sampling time

time = (1/60)*time; % Converts time to minutes

% Plots data
close all

figure(1)
plot(time, y1, 'Linewidth', 2);
hold on
plot(time, r1, '--r', 'Linewidth', 2);
hold off
ylim([34 38]);
legend('y_1: Temperature measurement', 'r_1: Temperature reference');
ylabel('Temperature [\circ C]');
xlabel('Time [min]');

figure(2)
plot(time, y2, 'Linewidth', 2);
hold on
plot(time, r2, '--r', 'Linewidth', 2);
hold off
ylim([20 24]);
legend('y_2: Light intensity measurement', 'r_2: Light intensity reference');
ylabel('Light intensity [-]');
xlabel('Time [min]');

figure(3)
plot(time, u1, 'r', 'Linewidth', 2);
hold on
plot(time, u2, 'b', 'Linewidth', 2);
plot(time, zeros(length(time),1), 'k');
plot(time, 5*ones(length(time),1), 'k')
hold off
ylim([-0.1 5.1]);
legend('u_1: Bulb', 'u_2: Fan and LED');
ylabel('Input [V]');
xlabel('Time [min]');

end

