%% Simulation of udaq using LQG and closed-loop optimal PID controller

% Operating points
ys = [36 22]';
us = [2 2 2]';

% Initialize controllers
if (~udaq_controller_2inputs_run)
    udaq_controllers_2inputs;  
end

minutes = 60;
simtime = 14*minutes;

r1 = [
     [0:Ts:4*minutes-Ts]'           zeros(length(0:Ts:4*minutes-Ts),1);
     [4*minutes:Ts:7*minutes-Ts]'   ones(length(4*minutes:Ts:7*minutes-Ts),1);
     [7*minutes:Ts:10*minutes-Ts]'   -1*ones(length(7*minutes:Ts:10*minutes-Ts),1);
     [10*minutes:Ts:14*minutes]'     zeros(length(10*minutes:Ts:14*minutes),1)
     ];
 
 r2 = [
     [0:Ts:4*minutes-Ts]'           zeros(length(0:Ts:4*minutes-Ts),1);
     [4*minutes:Ts:7*minutes-Ts]'   -1*ones(length(4*minutes:Ts:7*minutes-Ts),1);
     [7*minutes:Ts:10*minutes-Ts]'   ones(length(7*minutes:Ts:10*minutes-Ts),1);
     [10*minutes:Ts:14*minutes]'     zeros(length(10*minutes:Ts:14*minutes),1)
     ];

%% Plant warm-up
toggle = 0;         
LQG_switch = 0; % switch<=0: LQG, switch>0: PID
PID_switch = 0; % switch<=0: PID, switch>0: PI

%sim('udaq28LT_sim_2inputs.mdl', 5*minutes);

%% LQG control
toggle = 1; % turns on control
LQG_switch = 0; % switch<=0: LQG, switch>0: PID
PID_switch = 0; % switch<=0: PID, switch>0: PI

% Simulation with LQG control
sim('udaq28LT_sim_2inputs.mdl', simtime);

% Stores data
y1_lqg = y1.signals.values(:,2); % Temp inside system (direct)
y2_lqg = y2.signals.values(:,1);  % Light intensity (direct)
u1_lqg = u1.signals.values(:);
u2_lqg = u2.signals.values(:);
u3_lqg = u3.signals.values(:);
r1_lqg = r1(:,2)+ys(1);
r2_lqg = r2(:,2)+ys(2);
e_lqg = e.signals.values;
e1_lqg = e_lqg(1,:);
e2_lqg = e_lqg(2,:);

%% PID or PI control with or without low-pass filter (manual switches)
toggle = 1; % turns on control
LQG_switch = 1; % switch<=0: LQG, switch>0: PID
PID_switch = 0; % switch<=0: PID, switch>0: PI

% Simulation with PID control
sim('udaq28LT_sim_2inputs.mdl', simtime);

% Stores data
y1_pid = y1.signals.values(:,2); % Temp inside system (direct)
y2_pid = y2.signals.values(:,1);  % Light intensity (direct)
u1_pid = u1.signals.values(:);
u2_pid = u2.signals.values(:);
u3_pid = u3.signals.values(:);
r1_pid = r1(:,2)+ys(1);
r2_pid = r2(:,2)+ys(2);

%% PI control with or without low-pass filter
toggle = 1; % turns on control
LQG_switch = 1; % switch<=0: LQG, switch>0: PID
PID_switch = 1; % switch<=0: PID, switch>0: PI

% Simulation with PID control
sim('udaq28LT_sim_2inputs.mdl', simtime);

% Stores data
y1_pi = y1.signals.values(:,2); % Temp inside system (direct)
y2_pi = y2.signals.values(:,1);  % Light intensity (direct)
u1_pi = u1.signals.values(:);
u2_pi = u2.signals.values(:);
u3_pi = u3.signals.values(:);
r1_pi = r1(:,2)+ys(1);
r2_pi = r2(:,2)+ys(2);

%% Saves data
saveUdaqData('LQG_2inputs', y1_lqg, y2_lqg, u1_lqg, u2_lqg, u3_lqg, r1_lqg, r2_lqg, time);
saveUdaqData('PID_2inputs', y1_pid, y2_pid, u1_pid, u2_pid, u3_pid, r1_pid, r2_pid, time);
saveUdaqData('PI_2inputs', y1_pi, y2_pi, u1_pi, u2_pi, u3_pi, r1_pi, r2_pi, time);

%% Plots data
plotUdaqData2(y1_lqg, y2_lqg, u1_lqg, u2_lqg, r1_lqg, r2_lqg, time, Ts)
plotUdaqData2(y1_pid, y2_pid, u1_pid, u2_pid, r1_pid, r2_pid, time, Ts)
plotUdaqData2(y1_pi, y2_pi, u1_pi, u2_pi, r1_pi, r2_pi, time, Ts)

% Uncomment to plot loaded data
% plotUdaqData2(y1, y2, u1, u2, r1, r2, time, Ts)

% Plot estimation error for LQG
figure()
plot(time/60, e2_lqg, 'b', 'Linewidth', 1)
hold on 
plot(time/60, e1_lqg, 'k', 'Linewidth', 1)
hold off
ylim([-0.5 0.5]);
xlim([0 (1/60)*length(time)*Ts]);
legend('$e_2 = y_2 - \hat{y}_2$', '$e_1 = y_1 - \hat{y}_1$');
ylabel('Kalman filter error');
xlabel('Time [min]');

