function [ Juu,Jud ] = calcJuu( sysStruct, probStruct )
%CALCJUU Calculates Juu and Jud
%
%         |B^T*P*B+R        B^T*A^T*P*B    ...     B^T*(A^(N-1))^T*P*B|
%         |B^T*P*A*B        .                      B^T*(A^(N-2))^T*P*B|
% Juu/2 = |    :                         .                  :         |
%         |    .                                   .        .         |
%         |B^T*P*A^(N-1)*B  B^T*P*A^(N-2)*B    ...    B^T*P*B+R       |
%
%         |B'*P         |
% Jud/2 = |B'*P*A       |*A
%         |  :          |
%         |B'*P*A^(N-1) |

% Extract needed matrices
A = sysStruct.A;
B = sysStruct.B;
C = sysStruct.C;
Q = probStruct.Q;
R = probStruct.R;
P = probStruct.P;
N = probStruct.N;

[nx,nu,ny] = sysdim(A,B,C);

% Pre-calculation
Apot = zeros(nx, nx, N-1);
Apot(:,:,1) = A;
for h = 2:N-1
    Apot(:,:,h) = Apot(:,:,h-1)*A;
end

% Finds Ahat
Ahat = zeros(nx*(N-1),nu*N);
for i=1:N-1
    for j=1:i
        
        if (i == j)
            Ahat(nx*(i-1)+1:nx*i,nu*(j-1)+1:nu*j) = B;
        else
            Ahat(nx*(i-1)+1:nx*i,nu*(j-1)+1:nu*j) = Apot(:,:,i-j)*B;
        end
    end
end

% Finds a
a = zeros(nx*(N-1), nx);
a(1:nx,1:nx) = A;
for j = 2:N-1
    a(nx*(j-1)+1:nx*j, :) = Apot(:,:,j);
end

% Finds Qhat and Rhat
Qhat = blkdiag2(Q,N-1);
Rhat = blkdiag2(R,N);

% Finds Lambda matrix
L = zeros(nx, nu*(N));
L(:,nu*(N-1)+1:end) = B;
for k = linspace(N-1,1,N-1);
    L(:,nu*((N-1)-k)+1:nu*((N)-k)) = Apot(:,:,k)*B;
end

% Calculates Juu and Jud
Juu = 2*(Ahat'*Qhat*Ahat + L'*P*L + Rhat);
Jud = 2*(Ahat'*Qhat*a + L'*P'*(A^N));

end